# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal

import httpx

from ...types import (
    StreamCodec,
    StreamBidirectionalMode,
    StreamBidirectionalCodec,
    StreamBidirectionalTargetLegs,
    StreamBidirectionalSamplingRate,
    call_dial_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.stream_codec import StreamCodec
from ...types.sip_header_param import SipHeaderParam
from ...types.call_dial_response import CallDialResponse
from ...types.custom_sip_header_param import CustomSipHeaderParam
from ...types.dialogflow_config_param import DialogflowConfigParam
from ...types.sound_modifications_param import SoundModificationsParam
from ...types.stream_bidirectional_mode import StreamBidirectionalMode
from ...types.stream_bidirectional_codec import StreamBidirectionalCodec
from ...types.call_retrieve_status_response import CallRetrieveStatusResponse
from ...types.stream_bidirectional_target_legs import StreamBidirectionalTargetLegs
from ...types.stream_bidirectional_sampling_rate import StreamBidirectionalSamplingRate
from ...types.calls.transcription_start_request_param import TranscriptionStartRequestParam

__all__ = ["CallsResource", "AsyncCallsResource"]


class CallsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CallsResourceWithStreamingResponse(self)

    def dial(
        self,
        *,
        connection_id: str,
        from_: str,
        to: Union[str, SequenceNotStr[str]],
        answering_machine_detection: Literal[
            "premium", "detect", "detect_beep", "detect_words", "greeting_end", "disabled"
        ]
        | Omit = omit,
        answering_machine_detection_config: call_dial_params.AnsweringMachineDetectionConfig | Omit = omit,
        audio_url: str | Omit = omit,
        billing_group_id: str | Omit = omit,
        bridge_intent: bool | Omit = omit,
        bridge_on_answer: bool | Omit = omit,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        conference_config: call_dial_params.ConferenceConfig | Omit = omit,
        custom_headers: Iterable[CustomSipHeaderParam] | Omit = omit,
        dialogflow_config: DialogflowConfigParam | Omit = omit,
        enable_dialogflow: bool | Omit = omit,
        from_display_name: str | Omit = omit,
        link_to: str | Omit = omit,
        media_encryption: Literal["disabled", "SRTP", "DTLS"] | Omit = omit,
        media_name: str | Omit = omit,
        park_after_unbridge: str | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: Literal["record-from-answer"] | Omit = omit,
        record_channels: Literal["single", "dual"] | Omit = omit,
        record_custom_file_name: str | Omit = omit,
        record_format: Literal["wav", "mp3"] | Omit = omit,
        record_max_length: int | Omit = omit,
        record_timeout_secs: int | Omit = omit,
        record_track: Literal["both", "inbound", "outbound"] | Omit = omit,
        record_trim: Literal["trim-silence"] | Omit = omit,
        send_silence_when_idle: bool | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        sip_headers: Iterable[SipHeaderParam] | Omit = omit,
        sip_transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        sound_modifications: SoundModificationsParam | Omit = omit,
        stream_bidirectional_codec: StreamBidirectionalCodec | Omit = omit,
        stream_bidirectional_mode: StreamBidirectionalMode | Omit = omit,
        stream_bidirectional_sampling_rate: StreamBidirectionalSamplingRate | Omit = omit,
        stream_bidirectional_target_legs: StreamBidirectionalTargetLegs | Omit = omit,
        stream_codec: StreamCodec | Omit = omit,
        stream_establish_before_call_originate: bool | Omit = omit,
        stream_track: Literal["inbound_track", "outbound_track", "both_tracks"] | Omit = omit,
        stream_url: str | Omit = omit,
        supervise_call_control_id: str | Omit = omit,
        supervisor_role: Literal["barge", "whisper", "monitor"] | Omit = omit,
        time_limit_secs: int | Omit = omit,
        timeout_secs: int | Omit = omit,
        transcription: bool | Omit = omit,
        transcription_config: TranscriptionStartRequestParam | Omit = omit,
        webhook_url: str | Omit = omit,
        webhook_url_method: Literal["POST", "GET"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallDialResponse:
        """Dial a number or SIP URI from a given connection.

        A successful response will
        include a `call_leg_id` which can be used to correlate the command with
        subsequent webhooks.

        **Expected Webhooks:**

        - `call.initiated`
        - `call.answered` or `call.hangup`
        - `call.machine.detection.ended` if `answering_machine_detection` was requested
        - `call.machine.greeting.ended` if `answering_machine_detection` was requested
          to detect the end of machine greeting
        - `call.machine.premium.detection.ended` if
          `answering_machine_detection=premium` was requested
        - `call.machine.premium.greeting.ended` if `answering_machine_detection=premium`
          was requested and a beep was detected
        - `streaming.started`, `streaming.stopped` or `streaming.failed` if `stream_url`
          was set

        When the `record` parameter is set to `record-from-answer`, the response will
        include a `recording_id` field.

        Args:
          connection_id: The ID of the Call Control App (formerly ID of the connection) to be used when
              dialing the destination.

          from_: The `from` number to be used as the caller id presented to the destination (`to`
              number). The number should be in +E164 format.

          to: The DID or SIP URI to dial out to. Multiple DID or SIP URIs can be provided
              using an array of strings

          answering_machine_detection: Enables Answering Machine Detection. Telnyx offers Premium and Standard
              detections. With Premium detection, when a call is answered, Telnyx runs
              real-time detection and sends a `call.machine.premium.detection.ended` webhook
              with one of the following results: `human_residence`, `human_business`,
              `machine`, `silence` or `fax_detected`. If we detect a beep, we also send a
              `call.machine.premium.greeting.ended` webhook with the result of
              `beep_detected`. If we detect a beep before
              `call.machine.premium.detection.ended` we only send
              `call.machine.premium.greeting.ended`, and if we detect a beep after
              `call.machine.premium.detection.ended`, we send both webhooks. With Standard
              detection, when a call is answered, Telnyx runs real-time detection to determine
              if it was picked up by a human or a machine and sends an
              `call.machine.detection.ended` webhook with the analysis result. If
              `greeting_end` or `detect_words` is used and a `machine` is detected, you will
              receive another `call.machine.greeting.ended` webhook when the answering machine
              greeting ends with a beep or silence. If `detect_beep` is used, you will only
              receive `call.machine.greeting.ended` if a beep is detected.

          answering_machine_detection_config: Optional configuration parameters to modify 'answering_machine_detection'
              performance.

          audio_url: The URL of a file to be played back to the callee when the call is answered. The
              URL can point to either a WAV or MP3 file. media_name and audio_url cannot be
              used together in one request.

          billing_group_id: Use this field to set the Billing Group ID for the call. Must be a valid and
              existing Billing Group ID.

          bridge_intent: Indicates the intent to bridge this call with the call specified in link_to.
              When bridge_intent is true, link_to becomes required and the from number will be
              overwritten by the from number from the linked call.

          bridge_on_answer: Whether to automatically bridge answered call to the call specified in link_to.
              When bridge_on_answer is true, link_to becomes required.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore others Dial
              commands with the same `command_id`.

          conference_config: Optional configuration parameters to dial new participant into a conference.

          custom_headers: Custom headers to be added to the SIP INVITE.

          enable_dialogflow: Enables Dialogflow for the current call. The default value is false.

          from_display_name: The `from_display_name` string to be used as the caller id name (SIP From
              Display Name) presented to the destination (`to` number). The string should have
              a maximum of 128 characters, containing only letters, numbers, spaces, and
              -\\__~!.+ special characters. If ommited, the display name will be the same as the
              number in the `from` field.

          link_to: Use another call's control id for sharing the same call session id

          media_encryption: Defines whether media should be encrypted on the call.

          media_name: The media_name of a file to be played back to the callee when the call is
              answered. The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file.

          park_after_unbridge: If supplied with the value `self`, the current leg will be parked after
              unbridge. If not set, the default behavior is to hang up the leg. When
              park_after_unbridge is set, link_to becomes required.

          preferred_codecs: The list of comma-separated codecs in a preferred order for the forked media to
              be received.

          record: Start recording automatically after an event. Disabled by default.

          record_channels: Defines which channel should be recorded ('single' or 'dual') when `record` is
              specified.

          record_custom_file_name: The custom recording file name to be used instead of the default `call_leg_id`.
              Telnyx will still add a Unix timestamp suffix.

          record_format: Defines the format of the recording ('wav' or 'mp3') when `record` is specified.

          record_max_length: Defines the maximum length for the recording in seconds when `record` is
              specified. The minimum value is 0. The maximum value is 43200. The default value
              is 0 (infinite).

          record_timeout_secs: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected when `record` is specified. The timer only starts when the
              speech is detected. Please note that call transcription is used to detect
              silence and the related charge will be applied. The minimum value is 0. The
              default value is 0 (infinite).

          record_track: The audio track to be recorded. Can be either `both`, `inbound` or `outbound`.
              If only single track is specified (`inbound`, `outbound`), `channels`
              configuration is ignored and it will be recorded as mono (single channel).

          record_trim: When set to `trim-silence`, silence will be removed from the beginning and end
              of the recording.

          send_silence_when_idle: Generate silence RTP packets when no transmission available.

          sip_auth_password: SIP Authentication password used for SIP challenges.

          sip_auth_username: SIP Authentication username used for SIP challenges.

          sip_headers: SIP headers to be added to the SIP INVITE request. Currently only User-to-User
              header is supported.

          sip_transport_protocol: Defines SIP transport protocol to be used on the call.

          sound_modifications: Use this field to modify sound effects, for example adjust the pitch.

          stream_bidirectional_codec: Indicates codec for bidirectional streaming RTP payloads. Used only with
              stream_bidirectional_mode=rtp. Case sensitive.

          stream_bidirectional_mode: Configures method of bidirectional streaming (mp3, rtp).

          stream_bidirectional_sampling_rate: Audio sampling rate.

          stream_bidirectional_target_legs: Specifies which call legs should receive the bidirectional stream audio.

          stream_codec: Specifies the codec to be used for the streamed audio. When set to 'default' or
              when transcoding is not possible, the codec from the call will be used.

          stream_establish_before_call_originate: Establish websocket connection before dialing the destination. This is useful
              for cases where the websocket connection takes a long time to establish.

          stream_track: Specifies which track should be streamed.

          stream_url: The destination WebSocket address where the stream is going to be delivered.

          supervise_call_control_id: The call leg which will be supervised by the new call.

          supervisor_role: The role of the supervisor call. 'barge' means that supervisor call hears and is
              being heard by both ends of the call (caller & callee). 'whisper' means that
              only supervised_call_control_id hears supervisor but supervisor can hear
              everything. 'monitor' means that nobody can hear supervisor call, but supervisor
              can hear everything on the call.

          time_limit_secs: Sets the maximum duration of a Call Control Leg in seconds. If the time limit is
              reached, the call will hangup and a `call.hangup` webhook with a `hangup_cause`
              of `time_limit` will be sent. For example, by setting a time limit of 120
              seconds, a Call Leg will be automatically terminated two minutes after being
              answered. The default time limit is 14400 seconds or 4 hours and this is also
              the maximum allowed call length.

          timeout_secs: The number of seconds that Telnyx will wait for the call to be answered by the
              destination to which it is being called. If the timeout is reached before an
              answer is received, the call will hangup and a `call.hangup` webhook with a
              `hangup_cause` of `timeout` will be sent. Minimum value is 5 seconds. Maximum
              value is 600 seconds.

          transcription: Enable transcription upon call answer. The default value is false.

          webhook_url: Use this field to override the URL for which Telnyx will send subsequent
              webhooks to for this call.

          webhook_url_method: HTTP request type used for `webhook_url`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/calls",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "from_": from_,
                    "to": to,
                    "answering_machine_detection": answering_machine_detection,
                    "answering_machine_detection_config": answering_machine_detection_config,
                    "audio_url": audio_url,
                    "billing_group_id": billing_group_id,
                    "bridge_intent": bridge_intent,
                    "bridge_on_answer": bridge_on_answer,
                    "client_state": client_state,
                    "command_id": command_id,
                    "conference_config": conference_config,
                    "custom_headers": custom_headers,
                    "dialogflow_config": dialogflow_config,
                    "enable_dialogflow": enable_dialogflow,
                    "from_display_name": from_display_name,
                    "link_to": link_to,
                    "media_encryption": media_encryption,
                    "media_name": media_name,
                    "park_after_unbridge": park_after_unbridge,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "record_channels": record_channels,
                    "record_custom_file_name": record_custom_file_name,
                    "record_format": record_format,
                    "record_max_length": record_max_length,
                    "record_timeout_secs": record_timeout_secs,
                    "record_track": record_track,
                    "record_trim": record_trim,
                    "send_silence_when_idle": send_silence_when_idle,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "sip_headers": sip_headers,
                    "sip_transport_protocol": sip_transport_protocol,
                    "sound_modifications": sound_modifications,
                    "stream_bidirectional_codec": stream_bidirectional_codec,
                    "stream_bidirectional_mode": stream_bidirectional_mode,
                    "stream_bidirectional_sampling_rate": stream_bidirectional_sampling_rate,
                    "stream_bidirectional_target_legs": stream_bidirectional_target_legs,
                    "stream_codec": stream_codec,
                    "stream_establish_before_call_originate": stream_establish_before_call_originate,
                    "stream_track": stream_track,
                    "stream_url": stream_url,
                    "supervise_call_control_id": supervise_call_control_id,
                    "supervisor_role": supervisor_role,
                    "time_limit_secs": time_limit_secs,
                    "timeout_secs": timeout_secs,
                    "transcription": transcription,
                    "transcription_config": transcription_config,
                    "webhook_url": webhook_url,
                    "webhook_url_method": webhook_url_method,
                },
                call_dial_params.CallDialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallDialResponse,
        )

    def retrieve_status(
        self,
        call_control_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallRetrieveStatusResponse:
        """
        Returns the status of a call (data is available 10 minutes after call ended).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        return self._get(
            f"/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallRetrieveStatusResponse,
        )


class AsyncCallsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCallsResourceWithStreamingResponse(self)

    async def dial(
        self,
        *,
        connection_id: str,
        from_: str,
        to: Union[str, SequenceNotStr[str]],
        answering_machine_detection: Literal[
            "premium", "detect", "detect_beep", "detect_words", "greeting_end", "disabled"
        ]
        | Omit = omit,
        answering_machine_detection_config: call_dial_params.AnsweringMachineDetectionConfig | Omit = omit,
        audio_url: str | Omit = omit,
        billing_group_id: str | Omit = omit,
        bridge_intent: bool | Omit = omit,
        bridge_on_answer: bool | Omit = omit,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        conference_config: call_dial_params.ConferenceConfig | Omit = omit,
        custom_headers: Iterable[CustomSipHeaderParam] | Omit = omit,
        dialogflow_config: DialogflowConfigParam | Omit = omit,
        enable_dialogflow: bool | Omit = omit,
        from_display_name: str | Omit = omit,
        link_to: str | Omit = omit,
        media_encryption: Literal["disabled", "SRTP", "DTLS"] | Omit = omit,
        media_name: str | Omit = omit,
        park_after_unbridge: str | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: Literal["record-from-answer"] | Omit = omit,
        record_channels: Literal["single", "dual"] | Omit = omit,
        record_custom_file_name: str | Omit = omit,
        record_format: Literal["wav", "mp3"] | Omit = omit,
        record_max_length: int | Omit = omit,
        record_timeout_secs: int | Omit = omit,
        record_track: Literal["both", "inbound", "outbound"] | Omit = omit,
        record_trim: Literal["trim-silence"] | Omit = omit,
        send_silence_when_idle: bool | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        sip_headers: Iterable[SipHeaderParam] | Omit = omit,
        sip_transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        sound_modifications: SoundModificationsParam | Omit = omit,
        stream_bidirectional_codec: StreamBidirectionalCodec | Omit = omit,
        stream_bidirectional_mode: StreamBidirectionalMode | Omit = omit,
        stream_bidirectional_sampling_rate: StreamBidirectionalSamplingRate | Omit = omit,
        stream_bidirectional_target_legs: StreamBidirectionalTargetLegs | Omit = omit,
        stream_codec: StreamCodec | Omit = omit,
        stream_establish_before_call_originate: bool | Omit = omit,
        stream_track: Literal["inbound_track", "outbound_track", "both_tracks"] | Omit = omit,
        stream_url: str | Omit = omit,
        supervise_call_control_id: str | Omit = omit,
        supervisor_role: Literal["barge", "whisper", "monitor"] | Omit = omit,
        time_limit_secs: int | Omit = omit,
        timeout_secs: int | Omit = omit,
        transcription: bool | Omit = omit,
        transcription_config: TranscriptionStartRequestParam | Omit = omit,
        webhook_url: str | Omit = omit,
        webhook_url_method: Literal["POST", "GET"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallDialResponse:
        """Dial a number or SIP URI from a given connection.

        A successful response will
        include a `call_leg_id` which can be used to correlate the command with
        subsequent webhooks.

        **Expected Webhooks:**

        - `call.initiated`
        - `call.answered` or `call.hangup`
        - `call.machine.detection.ended` if `answering_machine_detection` was requested
        - `call.machine.greeting.ended` if `answering_machine_detection` was requested
          to detect the end of machine greeting
        - `call.machine.premium.detection.ended` if
          `answering_machine_detection=premium` was requested
        - `call.machine.premium.greeting.ended` if `answering_machine_detection=premium`
          was requested and a beep was detected
        - `streaming.started`, `streaming.stopped` or `streaming.failed` if `stream_url`
          was set

        When the `record` parameter is set to `record-from-answer`, the response will
        include a `recording_id` field.

        Args:
          connection_id: The ID of the Call Control App (formerly ID of the connection) to be used when
              dialing the destination.

          from_: The `from` number to be used as the caller id presented to the destination (`to`
              number). The number should be in +E164 format.

          to: The DID or SIP URI to dial out to. Multiple DID or SIP URIs can be provided
              using an array of strings

          answering_machine_detection: Enables Answering Machine Detection. Telnyx offers Premium and Standard
              detections. With Premium detection, when a call is answered, Telnyx runs
              real-time detection and sends a `call.machine.premium.detection.ended` webhook
              with one of the following results: `human_residence`, `human_business`,
              `machine`, `silence` or `fax_detected`. If we detect a beep, we also send a
              `call.machine.premium.greeting.ended` webhook with the result of
              `beep_detected`. If we detect a beep before
              `call.machine.premium.detection.ended` we only send
              `call.machine.premium.greeting.ended`, and if we detect a beep after
              `call.machine.premium.detection.ended`, we send both webhooks. With Standard
              detection, when a call is answered, Telnyx runs real-time detection to determine
              if it was picked up by a human or a machine and sends an
              `call.machine.detection.ended` webhook with the analysis result. If
              `greeting_end` or `detect_words` is used and a `machine` is detected, you will
              receive another `call.machine.greeting.ended` webhook when the answering machine
              greeting ends with a beep or silence. If `detect_beep` is used, you will only
              receive `call.machine.greeting.ended` if a beep is detected.

          answering_machine_detection_config: Optional configuration parameters to modify 'answering_machine_detection'
              performance.

          audio_url: The URL of a file to be played back to the callee when the call is answered. The
              URL can point to either a WAV or MP3 file. media_name and audio_url cannot be
              used together in one request.

          billing_group_id: Use this field to set the Billing Group ID for the call. Must be a valid and
              existing Billing Group ID.

          bridge_intent: Indicates the intent to bridge this call with the call specified in link_to.
              When bridge_intent is true, link_to becomes required and the from number will be
              overwritten by the from number from the linked call.

          bridge_on_answer: Whether to automatically bridge answered call to the call specified in link_to.
              When bridge_on_answer is true, link_to becomes required.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore others Dial
              commands with the same `command_id`.

          conference_config: Optional configuration parameters to dial new participant into a conference.

          custom_headers: Custom headers to be added to the SIP INVITE.

          enable_dialogflow: Enables Dialogflow for the current call. The default value is false.

          from_display_name: The `from_display_name` string to be used as the caller id name (SIP From
              Display Name) presented to the destination (`to` number). The string should have
              a maximum of 128 characters, containing only letters, numbers, spaces, and
              -\\__~!.+ special characters. If ommited, the display name will be the same as the
              number in the `from` field.

          link_to: Use another call's control id for sharing the same call session id

          media_encryption: Defines whether media should be encrypted on the call.

          media_name: The media_name of a file to be played back to the callee when the call is
              answered. The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file.

          park_after_unbridge: If supplied with the value `self`, the current leg will be parked after
              unbridge. If not set, the default behavior is to hang up the leg. When
              park_after_unbridge is set, link_to becomes required.

          preferred_codecs: The list of comma-separated codecs in a preferred order for the forked media to
              be received.

          record: Start recording automatically after an event. Disabled by default.

          record_channels: Defines which channel should be recorded ('single' or 'dual') when `record` is
              specified.

          record_custom_file_name: The custom recording file name to be used instead of the default `call_leg_id`.
              Telnyx will still add a Unix timestamp suffix.

          record_format: Defines the format of the recording ('wav' or 'mp3') when `record` is specified.

          record_max_length: Defines the maximum length for the recording in seconds when `record` is
              specified. The minimum value is 0. The maximum value is 43200. The default value
              is 0 (infinite).

          record_timeout_secs: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected when `record` is specified. The timer only starts when the
              speech is detected. Please note that call transcription is used to detect
              silence and the related charge will be applied. The minimum value is 0. The
              default value is 0 (infinite).

          record_track: The audio track to be recorded. Can be either `both`, `inbound` or `outbound`.
              If only single track is specified (`inbound`, `outbound`), `channels`
              configuration is ignored and it will be recorded as mono (single channel).

          record_trim: When set to `trim-silence`, silence will be removed from the beginning and end
              of the recording.

          send_silence_when_idle: Generate silence RTP packets when no transmission available.

          sip_auth_password: SIP Authentication password used for SIP challenges.

          sip_auth_username: SIP Authentication username used for SIP challenges.

          sip_headers: SIP headers to be added to the SIP INVITE request. Currently only User-to-User
              header is supported.

          sip_transport_protocol: Defines SIP transport protocol to be used on the call.

          sound_modifications: Use this field to modify sound effects, for example adjust the pitch.

          stream_bidirectional_codec: Indicates codec for bidirectional streaming RTP payloads. Used only with
              stream_bidirectional_mode=rtp. Case sensitive.

          stream_bidirectional_mode: Configures method of bidirectional streaming (mp3, rtp).

          stream_bidirectional_sampling_rate: Audio sampling rate.

          stream_bidirectional_target_legs: Specifies which call legs should receive the bidirectional stream audio.

          stream_codec: Specifies the codec to be used for the streamed audio. When set to 'default' or
              when transcoding is not possible, the codec from the call will be used.

          stream_establish_before_call_originate: Establish websocket connection before dialing the destination. This is useful
              for cases where the websocket connection takes a long time to establish.

          stream_track: Specifies which track should be streamed.

          stream_url: The destination WebSocket address where the stream is going to be delivered.

          supervise_call_control_id: The call leg which will be supervised by the new call.

          supervisor_role: The role of the supervisor call. 'barge' means that supervisor call hears and is
              being heard by both ends of the call (caller & callee). 'whisper' means that
              only supervised_call_control_id hears supervisor but supervisor can hear
              everything. 'monitor' means that nobody can hear supervisor call, but supervisor
              can hear everything on the call.

          time_limit_secs: Sets the maximum duration of a Call Control Leg in seconds. If the time limit is
              reached, the call will hangup and a `call.hangup` webhook with a `hangup_cause`
              of `time_limit` will be sent. For example, by setting a time limit of 120
              seconds, a Call Leg will be automatically terminated two minutes after being
              answered. The default time limit is 14400 seconds or 4 hours and this is also
              the maximum allowed call length.

          timeout_secs: The number of seconds that Telnyx will wait for the call to be answered by the
              destination to which it is being called. If the timeout is reached before an
              answer is received, the call will hangup and a `call.hangup` webhook with a
              `hangup_cause` of `timeout` will be sent. Minimum value is 5 seconds. Maximum
              value is 600 seconds.

          transcription: Enable transcription upon call answer. The default value is false.

          webhook_url: Use this field to override the URL for which Telnyx will send subsequent
              webhooks to for this call.

          webhook_url_method: HTTP request type used for `webhook_url`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/calls",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "from_": from_,
                    "to": to,
                    "answering_machine_detection": answering_machine_detection,
                    "answering_machine_detection_config": answering_machine_detection_config,
                    "audio_url": audio_url,
                    "billing_group_id": billing_group_id,
                    "bridge_intent": bridge_intent,
                    "bridge_on_answer": bridge_on_answer,
                    "client_state": client_state,
                    "command_id": command_id,
                    "conference_config": conference_config,
                    "custom_headers": custom_headers,
                    "dialogflow_config": dialogflow_config,
                    "enable_dialogflow": enable_dialogflow,
                    "from_display_name": from_display_name,
                    "link_to": link_to,
                    "media_encryption": media_encryption,
                    "media_name": media_name,
                    "park_after_unbridge": park_after_unbridge,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "record_channels": record_channels,
                    "record_custom_file_name": record_custom_file_name,
                    "record_format": record_format,
                    "record_max_length": record_max_length,
                    "record_timeout_secs": record_timeout_secs,
                    "record_track": record_track,
                    "record_trim": record_trim,
                    "send_silence_when_idle": send_silence_when_idle,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "sip_headers": sip_headers,
                    "sip_transport_protocol": sip_transport_protocol,
                    "sound_modifications": sound_modifications,
                    "stream_bidirectional_codec": stream_bidirectional_codec,
                    "stream_bidirectional_mode": stream_bidirectional_mode,
                    "stream_bidirectional_sampling_rate": stream_bidirectional_sampling_rate,
                    "stream_bidirectional_target_legs": stream_bidirectional_target_legs,
                    "stream_codec": stream_codec,
                    "stream_establish_before_call_originate": stream_establish_before_call_originate,
                    "stream_track": stream_track,
                    "stream_url": stream_url,
                    "supervise_call_control_id": supervise_call_control_id,
                    "supervisor_role": supervisor_role,
                    "time_limit_secs": time_limit_secs,
                    "timeout_secs": timeout_secs,
                    "transcription": transcription,
                    "transcription_config": transcription_config,
                    "webhook_url": webhook_url,
                    "webhook_url_method": webhook_url_method,
                },
                call_dial_params.CallDialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallDialResponse,
        )

    async def retrieve_status(
        self,
        call_control_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallRetrieveStatusResponse:
        """
        Returns the status of a call (data is available 10 minutes after call ended).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        return await self._get(
            f"/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallRetrieveStatusResponse,
        )


class CallsResourceWithRawResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.dial = to_raw_response_wrapper(
            calls.dial,
        )
        self.retrieve_status = to_raw_response_wrapper(
            calls.retrieve_status,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._calls.actions)


class AsyncCallsResourceWithRawResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.dial = async_to_raw_response_wrapper(
            calls.dial,
        )
        self.retrieve_status = async_to_raw_response_wrapper(
            calls.retrieve_status,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._calls.actions)


class CallsResourceWithStreamingResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.dial = to_streamed_response_wrapper(
            calls.dial,
        )
        self.retrieve_status = to_streamed_response_wrapper(
            calls.retrieve_status,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._calls.actions)


class AsyncCallsResourceWithStreamingResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.dial = async_to_streamed_response_wrapper(
            calls.dial,
        )
        self.retrieve_status = async_to_streamed_response_wrapper(
            calls.retrieve_status,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._calls.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .verify_profile import VerifyProfile
from .verifications.verify_meta import VerifyMeta

__all__ = ["VerifyProfileListResponse"]


class VerifyProfileListResponse(BaseModel):
    data: List[VerifyProfile]

    meta: VerifyMeta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["VerifyMeta"]


class VerifyMeta(BaseModel):
    page_number: Optional[int] = None

    page_size: Optional[int] = None

    total_pages: Optional[int] = None

    total_results: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from telnyx._utils import parse_date

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNumberLookup:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.create()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.create(
            aggregation_type="ALL",
            end_date=parse_date("2025-02-10"),
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            start_date=parse_date("2025-02-10"),
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.number_lookup.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.retrieve(
            "id",
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.number_lookup.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.usage_reports.number_lookup.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.list()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.list(
            page=0,
            per_page=0,
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.number_lookup.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        number_lookup = client.legacy.reporting.usage_reports.number_lookup.delete(
            "id",
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.number_lookup.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.usage_reports.number_lookup.with_raw_response.delete(
                "",
            )


class TestAsyncNumberLookup:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.create()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.create(
            aggregation_type="ALL",
            end_date=parse_date("2025-02-10"),
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            start_date=parse_date("2025-02-10"),
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = await response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with (
            async_client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.create()
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = await response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.retrieve(
            "id",
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = await response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = await response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.list()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.list(
            page=0,
            per_page=0,
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = await response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = await response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        number_lookup = await async_client.legacy.reporting.usage_reports.number_lookup.delete(
            "id",
        )
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_lookup = await response.parse()
        assert number_lookup is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.number_lookup.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_lookup = await response.parse()
            assert number_lookup is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.usage_reports.number_lookup.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from .phone_number_with_voice_settings import PhoneNumberWithVoiceSettings

__all__ = ["VoiceListResponse"]


class VoiceListResponse(BaseModel):
    data: Optional[List[PhoneNumberWithVoiceSettings]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RoomRecordingDeleteBulkParams", "Filter", "FilterDateEndedAt", "FilterDateStartedAt", "Page"]


class RoomRecordingDeleteBulkParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_ended_at][eq], filter[date_ended_at][gte],
    filter[date_ended_at][lte], filter[date_started_at][eq],
    filter[date_started_at][gte], filter[date_started_at][lte], filter[room_id],
    filter[participant_id], filter[session_id], filter[status], filter[type],
    filter[duration_secs]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterDateEndedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings ended on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings ended on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings ended on or before that date."""


class FilterDateStartedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings started on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings started on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room recordings started on or before that date."""


class Filter(TypedDict, total=False):
    date_ended_at: FilterDateEndedAt

    date_started_at: FilterDateStartedAt

    duration_secs: int
    """duration_secs greater or equal for filtering room recordings."""

    participant_id: str
    """participant_id for filtering room recordings."""

    room_id: str
    """room_id for filtering room recordings."""

    session_id: str
    """session_id for filtering room recordings."""

    status: str
    """status for filtering room recordings."""

    type: str
    """type for filtering room recordings."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["NotificationChannelCreateParams"]


class NotificationChannelCreateParams(TypedDict, total=False):
    channel_destination: str
    """The destination associated with the channel type."""

    channel_type_id: Literal["sms", "voice", "email", "webhook"]
    """A Channel Type ID"""

    notification_profile_id: str
    """A UUID reference to the associated Notification Profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .texml_recording_subresources_uris import TexmlRecordingSubresourcesUris

__all__ = ["TexmlGetCallRecordingResponseBody"]


class TexmlGetCallRecordingResponseBody(BaseModel):
    account_sid: Optional[str] = None

    call_sid: Optional[str] = None

    channels: Optional[Literal[1, 2]] = None

    conference_sid: Optional[str] = None

    date_created: Optional[datetime] = None

    date_updated: Optional[datetime] = None

    duration: Optional[str] = None
    """The duration of this recording, given in seconds."""

    error_code: Optional[str] = None

    media_url: Optional[str] = None

    sid: Optional[str] = None
    """Identifier of a resource."""

    source: Optional[
        Literal[
            "StartCallRecordingAPI",
            "StartConferenceRecordingAPI",
            "OutboundAPI",
            "DialVerb",
            "Conference",
            "RecordVerb",
            "Trunking",
        ]
    ] = None
    """Defines how the recording was created."""

    start_time: Optional[datetime] = None

    status: Optional[Literal["in-progress", "completed", "paused", "stopped"]] = None

    subresources_uris: Optional[TexmlRecordingSubresourcesUris] = None
    """Subresources details for a recording if available."""

    uri: Optional[str] = None
    """The relative URI for this recording resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    TelnyxBrand,
    BrandListResponse,
    BrandRetrieveResponse,
    BrandGetFeedbackResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBrand:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        brand = client.brand.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        brand = client.brand.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
            business_contact_email="name@example.com",
            city="New York",
            company_name="ABC Inc.",
            ein="111111111",
            first_name="John",
            ip_address="ipAddress",
            is_reseller=True,
            last_name="Smith",
            mobile_phone="+12024567890",
            mock=True,
            phone="+12024567890",
            postal_code="10001",
            state="NY",
            stock_exchange="NASDAQ",
            stock_symbol="ABC",
            street="123",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
            website="http://www.abcmobile.com",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(TelnyxBrand, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        brand = client.brand.retrieve(
            "brandId",
        )
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.retrieve(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.retrieve(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        brand = client.brand.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        brand = client.brand.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
            alt_business_id="altBusiness_id",
            alt_business_id_type="NONE",
            business_contact_email="name@example.com",
            city="New York",
            company_name="ABC Inc.",
            ein="111111111",
            first_name="John",
            identity_status="VERIFIED",
            ip_address="ipAddress",
            is_reseller=True,
            last_name="Smith",
            phone="+12024567890",
            postal_code="10001",
            state="NY",
            stock_exchange="NASDAQ",
            stock_symbol="ABC",
            street="123",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
            website="http://www.abcmobile.com",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(TelnyxBrand, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.update(
                brand_id="",
                country="US",
                display_name="ABC Mobile",
                email="email",
                entity_type="PRIVATE_PROFIT",
                vertical="TECHNOLOGY",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        brand = client.brand.list()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        brand = client.brand.list(
            brand_id="826ef77a-348c-445b-81a5-a9b13c68fbfe",
            country="country",
            display_name="displayName",
            entity_type="entityType",
            page=1,
            records_per_page=0,
            sort="assignedCampaignsCount",
            state="state",
            tcr_brand_id="BBAND1",
        )
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandListResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        brand = client.brand.delete(
            "brandId",
        )
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.delete(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.delete(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(object, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_feedback(self, client: Telnyx) -> None:
        brand = client.brand.get_feedback(
            "brandId",
        )
        assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_feedback(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.get_feedback(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_feedback(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.get_feedback(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_feedback(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.get_feedback(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_resend_2fa_email(self, client: Telnyx) -> None:
        brand = client.brand.resend_2fa_email(
            "brandId",
        )
        assert brand is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_resend_2fa_email(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.resend_2fa_email(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert brand is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_resend_2fa_email(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.resend_2fa_email(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert brand is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_resend_2fa_email(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.resend_2fa_email(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_revet(self, client: Telnyx) -> None:
        brand = client.brand.revet(
            "brandId",
        )
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_revet(self, client: Telnyx) -> None:
        response = client.brand.with_raw_response.revet(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = response.parse()
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_revet(self, client: Telnyx) -> None:
        with client.brand.with_streaming_response.revet(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = response.parse()
            assert_matches_type(object, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_revet(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            client.brand.with_raw_response.revet(
                "",
            )


class TestAsyncBrand:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
            business_contact_email="name@example.com",
            city="New York",
            company_name="ABC Inc.",
            ein="111111111",
            first_name="John",
            ip_address="ipAddress",
            is_reseller=True,
            last_name="Smith",
            mobile_phone="+12024567890",
            mock=True,
            phone="+12024567890",
            postal_code="10001",
            state="NY",
            stock_exchange="NASDAQ",
            stock_symbol="ABC",
            street="123",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
            website="http://www.abcmobile.com",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.create(
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(TelnyxBrand, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.retrieve(
            "brandId",
        )
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.retrieve(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.retrieve(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandRetrieveResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
            alt_business_id="altBusiness_id",
            alt_business_id_type="NONE",
            business_contact_email="name@example.com",
            city="New York",
            company_name="ABC Inc.",
            ein="111111111",
            first_name="John",
            identity_status="VERIFIED",
            ip_address="ipAddress",
            is_reseller=True,
            last_name="Smith",
            phone="+12024567890",
            postal_code="10001",
            state="NY",
            stock_exchange="NASDAQ",
            stock_symbol="ABC",
            street="123",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
            website="http://www.abcmobile.com",
        )
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(TelnyxBrand, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.update(
            brand_id="brandId",
            country="US",
            display_name="ABC Mobile",
            email="email",
            entity_type="PRIVATE_PROFIT",
            vertical="TECHNOLOGY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(TelnyxBrand, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.update(
                brand_id="",
                country="US",
                display_name="ABC Mobile",
                email="email",
                entity_type="PRIVATE_PROFIT",
                vertical="TECHNOLOGY",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.list()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.list(
            brand_id="826ef77a-348c-445b-81a5-a9b13c68fbfe",
            country="country",
            display_name="displayName",
            entity_type="entityType",
            page=1,
            records_per_page=0,
            sort="assignedCampaignsCount",
            state="state",
            tcr_brand_id="BBAND1",
        )
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandListResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandListResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.delete(
            "brandId",
        )
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.delete(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.delete(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(object, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_feedback(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.get_feedback(
            "brandId",
        )
        assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_feedback(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.get_feedback(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_feedback(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.get_feedback(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(BrandGetFeedbackResponse, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_feedback(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.get_feedback(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_resend_2fa_email(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.resend_2fa_email(
            "brandId",
        )
        assert brand is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_resend_2fa_email(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.resend_2fa_email(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert brand is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_resend_2fa_email(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.resend_2fa_email(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert brand is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_resend_2fa_email(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.resend_2fa_email(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_revet(self, async_client: AsyncTelnyx) -> None:
        brand = await async_client.brand.revet(
            "brandId",
        )
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_revet(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.brand.with_raw_response.revet(
            "brandId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        brand = await response.parse()
        assert_matches_type(object, brand, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_revet(self, async_client: AsyncTelnyx) -> None:
        async with async_client.brand.with_streaming_response.revet(
            "brandId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            brand = await response.parse()
            assert_matches_type(object, brand, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_revet(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `brand_id` but received ''"):
            await async_client.brand.with_raw_response.revet(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .available_service import AvailableService

__all__ = [
    "NetworkCoverageListParams",
    "Filter",
    "Filters",
    "FiltersAvailableServices",
    "FiltersAvailableServicesContains",
    "Page",
]


class NetworkCoverageListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[location.region], filter[location.site],
    filter[location.pop], filter[location.code]
    """

    filters: Filters
    """Consolidated filters parameter (deepObject style).

    Originally: filters[available_services][contains]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class Filter(TypedDict, total=False):
    location_code: Annotated[str, PropertyInfo(alias="location.code")]
    """The code of associated location to filter on."""

    location_pop: Annotated[str, PropertyInfo(alias="location.pop")]
    """The POP of associated location to filter on."""

    location_region: Annotated[str, PropertyInfo(alias="location.region")]
    """The region of associated location to filter on."""

    location_site: Annotated[str, PropertyInfo(alias="location.site")]
    """The site of associated location to filter on."""


class FiltersAvailableServicesContains(TypedDict, total=False):
    contains: AvailableService
    """Filter by available services containing the specified service"""


FiltersAvailableServices: TypeAlias = Union[AvailableService, FiltersAvailableServicesContains]


class Filters(TypedDict, total=False):
    available_services: FiltersAvailableServices
    """Filter by exact available service match"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["ConversationAddMessageParams"]


class ConversationAddMessageParams(TypedDict, total=False):
    role: Required[str]

    content: str

    metadata: Dict[str, Union[str, int, bool, SequenceNotStr[Union[str, int, bool]]]]

    name: str

    sent_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    tool_call_id: str

    tool_calls: Iterable[Dict[str, object]]

    tool_choice: Union[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .call_control_application_inbound import CallControlApplicationInbound
from .call_control_application_outbound import CallControlApplicationOutbound

__all__ = ["CallControlApplication"]


class CallControlApplication(BaseModel):
    id: Optional[str] = None

    active: Optional[bool] = None
    """Specifies whether the connection can be used."""

    anchorsite_override: Optional[Literal['"Latency"', '"Chicago, IL"', '"Ashburn, VA"', '"San Jose, CA"']] = None
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    application_name: Optional[str] = None
    """A user-assigned name to help manage the application."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was created"""

    dtmf_type: Optional[Literal["RFC 2833", "Inband", "SIP INFO"]] = None
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    first_command_timeout: Optional[bool] = None
    """
    Specifies whether calls to phone numbers associated with this connection should
    hangup after timing out.
    """

    first_command_timeout_secs: Optional[int] = None
    """Specifies how many seconds to wait before timing out a dial command."""

    inbound: Optional[CallControlApplicationInbound] = None

    outbound: Optional[CallControlApplicationOutbound] = None

    record_type: Optional[Literal["call_control_application"]] = None

    redact_dtmf_debug_logging: Optional[bool] = None
    """
    When enabled, DTMF digits entered by users will be redacted in debug logs to
    protect PII data entered through IVR interactions.
    """

    tags: Optional[List[str]] = None
    """Tags assigned to the Call Control Application."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was last updated"""

    webhook_api_version: Optional[Literal["1", "2"]] = None
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as `https`.
    """

    webhook_event_url: Optional[str] = None
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as `https`.
    """

    webhook_timeout_secs: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .inference_embedding_webhook_tool_params import InferenceEmbeddingWebhookToolParams

__all__ = ["WebhookTool"]


class WebhookTool(BaseModel):
    type: Literal["webhook"]

    webhook: InferenceEmbeddingWebhookToolParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["MessagingFeatureSet"]


class MessagingFeatureSet(BaseModel):
    domestic_two_way: bool
    """Send messages to and receive messages from numbers in the same country."""

    international_inbound: bool
    """Receive messages from numbers in other countries."""

    international_outbound: bool
    """Send messages to numbers in other countries."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["CallEventListResponse", "Data"]


class Data(BaseModel):
    call_leg_id: str
    """Uniquely identifies an individual call leg."""

    call_session_id: str
    """Uniquely identifies the call control session.

    A session may include multiple call leg events.
    """

    event_timestamp: str
    """Event timestamp"""

    metadata: object
    """
    Event metadata, which includes raw event, and extra information based on event
    type
    """

    name: str
    """Event name"""

    record_type: Literal["call_event"]

    type: Literal["command", "webhook"]
    """Event type"""


class CallEventListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import json
from typing import Mapping, cast
from datetime import datetime, timezone

from .._models import construct_type
from .._resource import SyncAPIResource, AsyncAPIResource
from .._exceptions import TelnyxError
from ..types.unwrap_webhook_event import UnwrapWebhookEvent
from ..types.unsafe_unwrap_webhook_event import UnsafeUnwrapWebhookEvent

__all__ = ["WebhooksResource", "AsyncWebhooksResource", "TelnyxWebhook", "TelnyxWebhookVerificationError"]


class TelnyxWebhookVerificationError(TelnyxError):
    """Raised when webhook verification fails."""
    pass


class TelnyxWebhook:
    """
    Telnyx webhook verification following the standardwebhooks pattern.
    
    This class provides ED25519 signature verification for Telnyx webhooks
    using the same interface pattern as the standardwebhooks library.
    """
    
    def __init__(self, key: str | bytes):
        """
        Initialize the webhook verifier with a public key.
        
        Args:
            key: The public key for verification (hex string or bytes)
        """
        try:
            from nacl.signing import VerifyKey
        except ImportError as exc:
            raise TelnyxError("You need to install `pynacl` to verify Telnyx webhooks") from exc
        
        # Convert key to bytes if it's a string
        if isinstance(key, str):
            try:
                key = bytes.fromhex(key)  # Convert from hex string to bytes
            except ValueError as exc:
                raise TelnyxWebhookVerificationError(f"Invalid key format: {key!r}") from exc
        
        self._verify_key = VerifyKey(key)
    
    def verify(self, payload: str, headers: Mapping[str, str]) -> None:
        """
        Verify a webhook payload and headers.
        
        Args:
            payload: The webhook payload string
            headers: The webhook headers
            
        Raises:
            TelnyxWebhookVerificationError: If verification fails
        """
        try:
            from nacl.exceptions import BadSignatureError
        except ImportError as exc:
            raise TelnyxError("You need to install `pynacl` to verify Telnyx webhooks") from exc
        
        # Extract required headers (case-insensitive lookup)
        signature_header = headers.get("Telnyx-Signature-Ed25519") or headers.get("telnyx-signature-ed25519")
        timestamp_header = headers.get("Telnyx-Timestamp") or headers.get("telnyx-timestamp")
        user_agent = headers.get("User-Agent") or headers.get("user-agent", "")

        # Validate required headers
        if not signature_header:
            raise TelnyxWebhookVerificationError("Missing required header: Telnyx-Signature-Ed25519")
        
        if not timestamp_header:
            raise TelnyxWebhookVerificationError("Missing required header: Telnyx-Timestamp")

        # Verify User-Agent if present (optional security check)
        if user_agent and "telnyx-webhooks" not in user_agent.lower():
            raise TelnyxWebhookVerificationError(f"Unexpected User-Agent: {user_agent}")

        # Validate timestamp format and prevent replay attacks
        try:
            webhook_time = int(timestamp_header)
            current_time = int(datetime.now(timezone.utc).timestamp())
            
            # Allow 5 minutes tolerance
            if abs(current_time - webhook_time) > 300:
                raise TelnyxWebhookVerificationError(
                    f"Webhook timestamp too old or too new: {timestamp_header}"
                )
        except ValueError as exc:
            raise TelnyxWebhookVerificationError(
                f"Invalid timestamp format: {timestamp_header}"
            ) from exc

        # Decode the signature from hex
        try:
            signature = bytes.fromhex(signature_header)
        except ValueError as exc:
            raise TelnyxWebhookVerificationError(
                f"Invalid signature format: {signature_header}"
            ) from exc

        # Create the signed payload: timestamp + payload
        signed_payload = timestamp_header.encode('utf-8') + payload.encode('utf-8')

        # Verify the signature
        try:
            self._verify_key.verify(signed_payload, signature)
        except BadSignatureError as exc:
            raise TelnyxWebhookVerificationError("Invalid webhook signature") from exc


class WebhooksResource(SyncAPIResource):
    def unsafe_unwrap(self, payload: str) -> UnsafeUnwrapWebhookEvent:
        return cast(
            UnsafeUnwrapWebhookEvent,
            construct_type(
                type_=UnsafeUnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )

    def unwrap(self, payload: str, *, headers: Mapping[str, str], key: str | bytes | None = None) -> UnwrapWebhookEvent:
        if key is None:
            key = self._client.public_key
            if key is None:
                raise ValueError(
                    "Cannot verify a webhook without a key on either the client's public_key or passed in as an argument"
                )

        if not isinstance(headers, dict):
            headers = dict(headers)

        # Use Telnyx-specific webhook verification following standardwebhooks pattern
        webhook = TelnyxWebhook(key)
        webhook.verify(payload, headers)

        return cast(
            UnwrapWebhookEvent,
            construct_type(
                type_=UnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )


class AsyncWebhooksResource(AsyncAPIResource):
    def unsafe_unwrap(self, payload: str) -> UnsafeUnwrapWebhookEvent:
        return cast(
            UnsafeUnwrapWebhookEvent,
            construct_type(
                type_=UnsafeUnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )

    def unwrap(self, payload: str, *, headers: Mapping[str, str], key: str | bytes | None = None) -> UnwrapWebhookEvent:
        if key is None:
            key = self._client.public_key
            if key is None:
                raise ValueError(
                    "Cannot verify a webhook without a key on either the client's public_key or passed in as an argument"
                )

        if not isinstance(headers, dict):
            headers = dict(headers)

        # Use Telnyx-specific webhook verification following standardwebhooks pattern
        webhook = TelnyxWebhook(key)
        webhook.verify(payload, headers)

        return cast(
            UnwrapWebhookEvent,
            construct_type(
                type_=UnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .hangup_tool_params_param import HangupToolParamsParam

__all__ = ["HangupToolParam"]


class HangupToolParam(TypedDict, total=False):
    hangup: Required[HangupToolParamsParam]

    type: Required[Literal["hangup"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .version_config_param import VersionConfigParam

__all__ = ["CanaryDeployUpdateParams"]


class CanaryDeployUpdateParams(TypedDict, total=False):
    versions: Required[Iterable[VersionConfigParam]]
    """List of version configurations"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cnam_listing import CnamListing as CnamListing
from .csv_download import CsvDownload as CsvDownload
from .call_recording import CallRecording as CallRecording
from .media_features import MediaFeatures as MediaFeatures
from .call_forwarding import CallForwarding as CallForwarding
from .job_list_params import JobListParams as JobListParams
from .job_list_response import JobListResponse as JobListResponse
from .phone_numbers_job import PhoneNumbersJob as PhoneNumbersJob
from .voice_list_params import VoiceListParams as VoiceListParams
from .cnam_listing_param import CnamListingParam as CnamListingParam
from .voice_list_response import VoiceListResponse as VoiceListResponse
from .voice_update_params import VoiceUpdateParams as VoiceUpdateParams
from .call_recording_param import CallRecordingParam as CallRecordingParam
from .media_features_param import MediaFeaturesParam as MediaFeaturesParam
from .call_forwarding_param import CallForwardingParam as CallForwardingParam
from .job_retrieve_response import JobRetrieveResponse as JobRetrieveResponse
from .messaging_list_params import MessagingListParams as MessagingListParams
from .voice_update_response import VoiceUpdateResponse as VoiceUpdateResponse
from .job_delete_batch_params import JobDeleteBatchParams as JobDeleteBatchParams
from .job_update_batch_params import JobUpdateBatchParams as JobUpdateBatchParams
from .messaging_list_response import MessagingListResponse as MessagingListResponse
from .messaging_update_params import MessagingUpdateParams as MessagingUpdateParams
from .voice_retrieve_response import VoiceRetrieveResponse as VoiceRetrieveResponse
from .voicemail_create_params import VoicemailCreateParams as VoicemailCreateParams
from .voicemail_pref_response import VoicemailPrefResponse as VoicemailPrefResponse
from .voicemail_update_params import VoicemailUpdateParams as VoicemailUpdateParams
from .csv_download_list_params import CsvDownloadListParams as CsvDownloadListParams
from .job_delete_batch_response import JobDeleteBatchResponse as JobDeleteBatchResponse
from .job_update_batch_response import JobUpdateBatchResponse as JobUpdateBatchResponse
from .messaging_update_response import MessagingUpdateResponse as MessagingUpdateResponse
from .voicemail_create_response import VoicemailCreateResponse as VoicemailCreateResponse
from .voicemail_update_response import VoicemailUpdateResponse as VoicemailUpdateResponse
from .csv_download_create_params import CsvDownloadCreateParams as CsvDownloadCreateParams
from .csv_download_list_response import CsvDownloadListResponse as CsvDownloadListResponse
from .messaging_retrieve_response import MessagingRetrieveResponse as MessagingRetrieveResponse
from .update_voice_settings_param import UpdateVoiceSettingsParam as UpdateVoiceSettingsParam
from .voicemail_retrieve_response import VoicemailRetrieveResponse as VoicemailRetrieveResponse
from .csv_download_create_response import CsvDownloadCreateResponse as CsvDownloadCreateResponse
from .action_enable_emergency_params import ActionEnableEmergencyParams as ActionEnableEmergencyParams
from .action_verify_ownership_params import ActionVerifyOwnershipParams as ActionVerifyOwnershipParams
from .csv_download_retrieve_response import CsvDownloadRetrieveResponse as CsvDownloadRetrieveResponse
from .action_enable_emergency_response import ActionEnableEmergencyResponse as ActionEnableEmergencyResponse
from .action_verify_ownership_response import ActionVerifyOwnershipResponse as ActionVerifyOwnershipResponse
from .phone_number_with_voice_settings import PhoneNumberWithVoiceSettings as PhoneNumberWithVoiceSettings
from .action_change_bundle_status_params import ActionChangeBundleStatusParams as ActionChangeBundleStatusParams
from .action_change_bundle_status_response import ActionChangeBundleStatusResponse as ActionChangeBundleStatusResponse
from .job_update_emergency_settings_batch_params import (
    JobUpdateEmergencySettingsBatchParams as JobUpdateEmergencySettingsBatchParams,
)
from .job_update_emergency_settings_batch_response import (
    JobUpdateEmergencySettingsBatchResponse as JobUpdateEmergencySettingsBatchResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .porting_order_type import PortingOrderType

__all__ = [
    "PortingOrderListParams",
    "Filter",
    "FilterActivationSettings",
    "FilterActivationSettingsFocDatetimeRequested",
    "FilterEndUser",
    "FilterEndUserAdmin",
    "FilterMisc",
    "FilterPhoneNumbers",
    "FilterPhoneNumbersPhoneNumber",
    "Page",
    "Sort",
]


class PortingOrderListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[customer_reference], filter[customer_group_reference],
    filter[parent_support_key], filter[phone_numbers.country_code],
    filter[phone_numbers.carrier_name], filter[misc.type],
    filter[end_user.admin.entity_name], filter[end_user.admin.auth_person_name],
    filter[activation_settings.fast_port_eligible],
    filter[activation_settings.foc_datetime_requested][gt],
    filter[activation_settings.foc_datetime_requested][lt],
    filter[phone_numbers.phone_number][contains]
    """

    include_phone_numbers: bool
    """Include the first 50 phone number objects in the results"""

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class FilterActivationSettingsFocDatetimeRequested(TypedDict, total=False):
    gt: str
    """Filter results by foc date later than this value"""

    lt: str
    """Filter results by foc date earlier than this value"""


class FilterActivationSettings(TypedDict, total=False):
    fast_port_eligible: bool
    """Filter results by fast port eligible"""

    foc_datetime_requested: FilterActivationSettingsFocDatetimeRequested
    """FOC datetime range filtering operations"""


class FilterEndUserAdmin(TypedDict, total=False):
    auth_person_name: str
    """Filter results by authorized person"""

    entity_name: str
    """Filter results by person or company name"""


class FilterEndUser(TypedDict, total=False):
    admin: FilterEndUserAdmin


class FilterMisc(TypedDict, total=False):
    type: PortingOrderType
    """Filter results by porting order type"""


class FilterPhoneNumbersPhoneNumber(TypedDict, total=False):
    contains: str
    """Filter results by full or partial phone_number"""


class FilterPhoneNumbers(TypedDict, total=False):
    carrier_name: str
    """Filter results by old service provider"""

    country_code: str
    """Filter results by country ISO 3166-1 alpha-2 code"""

    phone_number: FilterPhoneNumbersPhoneNumber
    """Phone number pattern filtering operations"""


class Filter(TypedDict, total=False):
    activation_settings: FilterActivationSettings

    customer_group_reference: str
    """Filter results by customer_group_reference"""

    customer_reference: str
    """Filter results by customer_reference"""

    end_user: FilterEndUser

    misc: FilterMisc

    parent_support_key: str
    """Filter results by parent_support_key"""

    phone_numbers: FilterPhoneNumbers


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""


class Sort(TypedDict, total=False):
    value: Literal[
        "created_at",
        "-created_at",
        "activation_settings.foc_datetime_requested",
        "-activation_settings.foc_datetime_requested",
    ]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["AdvancedOrderUpdateRequirementGroupParams"]


class AdvancedOrderUpdateRequirementGroupParams(TypedDict, total=False):
    area_code: str

    comments: str

    country_code: str

    customer_reference: str

    features: List[Literal["sms", "mms", "voice", "fax", "emergency"]]

    phone_number_type: Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"]

    quantity: int

    requirement_group_id: str
    """The ID of the requirement group to associate with this advanced order"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .porting_order_misc_param import PortingOrderMiscParam
from .porting_order_end_user_param import PortingOrderEndUserParam
from .porting_order_documents_param import PortingOrderDocumentsParam
from .porting_order_user_feedback_param import PortingOrderUserFeedbackParam
from .porting_order_phone_number_configuration_param import PortingOrderPhoneNumberConfigurationParam

__all__ = ["PortingOrderUpdateParams", "ActivationSettings", "Messaging", "Requirement"]


class PortingOrderUpdateParams(TypedDict, total=False):
    activation_settings: ActivationSettings

    customer_group_reference: str

    customer_reference: str

    documents: PortingOrderDocumentsParam
    """Can be specified directly or via the `requirement_group_id` parameter."""

    end_user: PortingOrderEndUserParam

    messaging: Messaging

    misc: PortingOrderMiscParam

    phone_number_configuration: PortingOrderPhoneNumberConfigurationParam

    requirement_group_id: str
    """
    If present, we will read the current values from the specified Requirement Group
    into the Documents and Requirements for this Porting Order. Note that any future
    changes in the Requirement Group would have no impact on this Porting Order. We
    will return an error if a specified Requirement Group conflicts with documents
    or requirements in the same request.
    """

    requirements: Iterable[Requirement]
    """List of requirements for porting numbers."""

    user_feedback: PortingOrderUserFeedbackParam

    webhook_url: str


class ActivationSettings(TypedDict, total=False):
    foc_datetime_requested: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 formatted Date/Time requested for the FOC date"""


class Messaging(TypedDict, total=False):
    enable_messaging: bool
    """
    Indicates whether Telnyx will port messaging capabilities from the losing
    carrier. If false, any messaging capabilities will stay with their current
    provider.
    """


class Requirement(TypedDict, total=False):
    field_value: Required[str]
    """
    identifies the document or provides the text value that satisfies this
    requirement
    """

    requirement_type_id: Required[str]
    """Identifies the requirement type that the `field_value` fulfills"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import DocumentLinkListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocumentLinks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        document_link = client.document_links.list()
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        document_link = client.document_links.list(
            filter={
                "linked_record_type": "porting_order",
                "linked_resource_id": "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.document_links.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_link = response.parse()
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.document_links.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_link = response.parse()
            assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDocumentLinks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        document_link = await async_client.document_links.list()
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        document_link = await async_client.document_links.list(
            filter={
                "linked_record_type": "porting_order",
                "linked_resource_id": "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.document_links.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document_link = await response.parse()
        assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.document_links.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document_link = await response.parse()
            assert_matches_type(DocumentLinkListResponse, document_link, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta

__all__ = ["VerificationCodeListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this porting verification code"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    phone_number: Optional[str] = None
    """E164 formatted phone number"""

    porting_order_id: Optional[str] = None
    """Identifies the associated porting order"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    verified: Optional[bool] = None
    """Indicates whether the verification code has been verified"""


class VerificationCodeListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["PortingOrderCreateParams"]


class PortingOrderCreateParams(TypedDict, total=False):
    phone_numbers: Required[SequenceNotStr[str]]
    """The list of +E.164 formatted phone numbers"""

    customer_group_reference: str
    """A customer-specified group reference for customer bookkeeping purposes"""

    customer_reference: str
    """A customer-specified reference number for customer bookkeeping purposes"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionEnableEmergencyParams"]


class ActionEnableEmergencyParams(TypedDict, total=False):
    emergency_address_id: Required[str]
    """Identifies the address to be used with emergency services."""

    emergency_enabled: Required[bool]
    """Indicates whether to enable emergency services on this number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ActionRefreshResponse"]


class ActionRefreshResponse(BaseModel):
    result: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DialogflowConnectionCreateParams"]


class DialogflowConnectionCreateParams(TypedDict, total=False):
    service_account: Required[Dict[str, object]]
    """The JSON map to connect your Dialoglow account."""

    conversation_profile_id: str
    """The id of a configured conversation profile on your Dialogflow account.

    (If you use Dialogflow CX, this param is required)
    """

    dialogflow_api: Literal["cx", "es"]
    """Determine which Dialogflow will be used."""

    environment: str
    """Which Dialogflow environment will be used."""

    location: str
    """The region of your agent is. (If you use Dialogflow CX, this param is required)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record
from .interface_status import InterfaceStatus

__all__ = ["GlobalIPAssignment"]


class GlobalIPAssignment(Record):
    global_ip_id: Optional[str] = None
    """Global IP ID."""

    is_announced: Optional[bool] = None
    """Status of BGP announcement."""

    is_connected: Optional[bool] = None
    """Wireguard peer is connected."""

    is_in_maintenance: Optional[bool] = None
    """Enable/disable BGP announcement."""

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""

    status: Optional[InterfaceStatus] = None
    """The current status of the interface deployment."""

    wireguard_peer_id: Optional[str] = None
    """Wireguard peer ID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["NumberBlockOrderListParams", "Filter", "FilterCreatedAt", "Page"]


class NumberBlockOrderListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status], filter[created_at],
    filter[phone_numbers.starting_number]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterCreatedAt(TypedDict, total=False):
    gt: str
    """Filter number block orders later than this value."""

    lt: str
    """Filter number block orders earlier than this value."""


class Filter(TypedDict, total=False):
    created_at: FilterCreatedAt
    """Filter number block orders by date range."""

    phone_numbers_starting_number: Annotated[str, PropertyInfo(alias="phone_numbers.starting_number")]
    """Filter number block orders having these phone numbers."""

    status: str
    """Filter number block orders by status."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting.filter_param import FilterParam
from .....types.legacy.reporting.batch_detail_records import voice_create_params
from .....types.legacy.reporting.batch_detail_records.voice_list_response import VoiceListResponse
from .....types.legacy.reporting.batch_detail_records.voice_create_response import VoiceCreateResponse
from .....types.legacy.reporting.batch_detail_records.voice_delete_response import VoiceDeleteResponse
from .....types.legacy.reporting.batch_detail_records.voice_retrieve_response import VoiceRetrieveResponse
from .....types.legacy.reporting.batch_detail_records.voice_retrieve_fields_response import VoiceRetrieveFieldsResponse

__all__ = ["VoiceResource", "AsyncVoiceResource"]


class VoiceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoiceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        call_types: Iterable[int] | Omit = omit,
        connections: Iterable[int] | Omit = omit,
        fields: SequenceNotStr[str] | Omit = omit,
        filters: Iterable[FilterParam] | Omit = omit,
        include_all_metadata: bool | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        record_types: Iterable[int] | Omit = omit,
        report_name: str | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        source: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCreateResponse:
        """
        Creates a new CDR report request with the specified filters

        Args:
          end_time: End time in ISO format

          start_time: Start time in ISO format

          call_types: List of call types to filter by (Inbound = 1, Outbound = 2)

          connections: List of connections to filter by

          fields: Set of fields to include in the report

          filters: List of filters to apply

          include_all_metadata: Whether to include all metadata

          managed_accounts: List of managed accounts to include

          record_types: List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)

          report_name: Name of the report

          select_all_managed_accounts: Whether to select all managed accounts

          source: Source of the report. Valid values: calls (default), call-control, fax-api,
              webrtc

          timezone: Timezone for the report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/batch_detail_records/voice",
            body=maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "call_types": call_types,
                    "connections": connections,
                    "fields": fields,
                    "filters": filters,
                    "include_all_metadata": include_all_metadata,
                    "managed_accounts": managed_accounts,
                    "record_types": record_types,
                    "report_name": report_name,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "source": source,
                    "timezone": timezone,
                },
                voice_create_params.VoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Retrieves a specific CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/batch_detail_records/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceListResponse:
        """Retrieves all CDR report requests for the authenticated user"""
        return self._get(
            "/legacy/reporting/batch_detail_records/voice",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDeleteResponse:
        """
        Deletes a specific CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/legacy/reporting/batch_detail_records/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )

    def retrieve_fields(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveFieldsResponse:
        """Retrieves all available fields that can be used in CDR reports"""
        return self._get(
            "/legacy/reporting/batch_detail_records/voice/fields",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveFieldsResponse,
        )


class AsyncVoiceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoiceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        call_types: Iterable[int] | Omit = omit,
        connections: Iterable[int] | Omit = omit,
        fields: SequenceNotStr[str] | Omit = omit,
        filters: Iterable[FilterParam] | Omit = omit,
        include_all_metadata: bool | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        record_types: Iterable[int] | Omit = omit,
        report_name: str | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        source: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCreateResponse:
        """
        Creates a new CDR report request with the specified filters

        Args:
          end_time: End time in ISO format

          start_time: Start time in ISO format

          call_types: List of call types to filter by (Inbound = 1, Outbound = 2)

          connections: List of connections to filter by

          fields: Set of fields to include in the report

          filters: List of filters to apply

          include_all_metadata: Whether to include all metadata

          managed_accounts: List of managed accounts to include

          record_types: List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)

          report_name: Name of the report

          select_all_managed_accounts: Whether to select all managed accounts

          source: Source of the report. Valid values: calls (default), call-control, fax-api,
              webrtc

          timezone: Timezone for the report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/batch_detail_records/voice",
            body=await async_maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "call_types": call_types,
                    "connections": connections,
                    "fields": fields,
                    "filters": filters,
                    "include_all_metadata": include_all_metadata,
                    "managed_accounts": managed_accounts,
                    "record_types": record_types,
                    "report_name": report_name,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "source": source,
                    "timezone": timezone,
                },
                voice_create_params.VoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Retrieves a specific CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/batch_detail_records/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceListResponse:
        """Retrieves all CDR report requests for the authenticated user"""
        return await self._get(
            "/legacy/reporting/batch_detail_records/voice",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceDeleteResponse:
        """
        Deletes a specific CDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/legacy/reporting/batch_detail_records/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )

    async def retrieve_fields(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveFieldsResponse:
        """Retrieves all available fields that can be used in CDR reports"""
        return await self._get(
            "/legacy/reporting/batch_detail_records/voice/fields",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveFieldsResponse,
        )


class VoiceResourceWithRawResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.create = to_raw_response_wrapper(
            voice.create,
        )
        self.retrieve = to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = to_raw_response_wrapper(
            voice.list,
        )
        self.delete = to_raw_response_wrapper(
            voice.delete,
        )
        self.retrieve_fields = to_raw_response_wrapper(
            voice.retrieve_fields,
        )


class AsyncVoiceResourceWithRawResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.create = async_to_raw_response_wrapper(
            voice.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            voice.list,
        )
        self.delete = async_to_raw_response_wrapper(
            voice.delete,
        )
        self.retrieve_fields = async_to_raw_response_wrapper(
            voice.retrieve_fields,
        )


class VoiceResourceWithStreamingResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.create = to_streamed_response_wrapper(
            voice.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            voice.list,
        )
        self.delete = to_streamed_response_wrapper(
            voice.delete,
        )
        self.retrieve_fields = to_streamed_response_wrapper(
            voice.retrieve_fields,
        )


class AsyncVoiceResourceWithStreamingResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.create = async_to_streamed_response_wrapper(
            voice.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            voice.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            voice.delete,
        )
        self.retrieve_fields = async_to_streamed_response_wrapper(
            voice.retrieve_fields,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["PortingOrderPhoneNumberConfigurationParam"]


class PortingOrderPhoneNumberConfigurationParam(TypedDict, total=False):
    billing_group_id: str
    """identifies the billing group to set on the numbers when ported"""

    connection_id: str
    """identifies the connection to set on the numbers when ported"""

    emergency_address_id: str
    """identifies the emergency address to set on the numbers when ported"""

    messaging_profile_id: str
    """identifies the messaging profile to set on the numbers when ported"""

    tags: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .integration_secret import IntegrationSecret

__all__ = ["IntegrationSecretListResponse", "Meta"]


class Meta(BaseModel):
    page_number: int

    page_size: int

    total_pages: int

    total_results: int


class IntegrationSecretListResponse(BaseModel):
    data: List[IntegrationSecret]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .osr import (
    OsrResource,
    AsyncOsrResource,
    OsrResourceWithRawResponse,
    AsyncOsrResourceWithRawResponse,
    OsrResourceWithStreamingResponse,
    AsyncOsrResourceWithStreamingResponse,
)
from .usecase import (
    UsecaseResource,
    AsyncUsecaseResource,
    UsecaseResourceWithRawResponse,
    AsyncUsecaseResourceWithRawResponse,
    UsecaseResourceWithStreamingResponse,
    AsyncUsecaseResourceWithStreamingResponse,
)
from .campaign import (
    CampaignResource,
    AsyncCampaignResource,
    CampaignResourceWithRawResponse,
    AsyncCampaignResourceWithRawResponse,
    CampaignResourceWithStreamingResponse,
    AsyncCampaignResourceWithStreamingResponse,
)

__all__ = [
    "UsecaseResource",
    "AsyncUsecaseResource",
    "UsecaseResourceWithRawResponse",
    "AsyncUsecaseResourceWithRawResponse",
    "UsecaseResourceWithStreamingResponse",
    "AsyncUsecaseResourceWithStreamingResponse",
    "OsrResource",
    "AsyncOsrResource",
    "OsrResourceWithRawResponse",
    "AsyncOsrResourceWithRawResponse",
    "OsrResourceWithStreamingResponse",
    "AsyncOsrResourceWithStreamingResponse",
    "CampaignResource",
    "AsyncCampaignResource",
    "CampaignResourceWithRawResponse",
    "AsyncCampaignResourceWithRawResponse",
    "CampaignResourceWithStreamingResponse",
    "AsyncCampaignResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CallRetrieveResponse", "Data"]


class Data(BaseModel):
    call_control_id: str
    """Unique identifier and token for controlling the call."""

    call_leg_id: str
    """ID that is unique to the call and can be used to correlate webhook events"""

    call_session_id: str
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call
    """

    connection_id: str
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    enqueued_at: str
    """ISO 8601 formatted date of when the call was put in the queue"""

    from_: str = FieldInfo(alias="from")
    """Number or SIP URI placing the call."""

    queue_id: str
    """Unique identifier of the queue the call is in."""

    queue_position: int
    """Current position of the call in the queue"""

    record_type: Literal["queue_call"]

    to: str
    """Destination number or SIP URI of the call."""

    wait_time_secs: int
    """The time the call has been waiting in the queue, given in seconds"""


class CallRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from .update_voice_settings_param import UpdateVoiceSettingsParam

__all__ = ["JobUpdateBatchParams", "Filter", "FilterVoiceConnectionName"]


class JobUpdateBatchParams(TypedDict, total=False):
    phone_numbers: Required[SequenceNotStr[str]]
    """Array of phone number ids and/or phone numbers in E164 format to update.

    This parameter is required if no filter parameters are provided. If you want to
    update specific numbers rather than all numbers matching a filter, you must use
    this parameter. Each item must be either a valid phone number ID or a phone
    number in E164 format (e.g., '+13127367254').
    """

    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[has_bundle], filter[tag], filter[connection_id],
    filter[phone_number], filter[status], filter[voice.connection_name],
    filter[voice.usage_payment_method], filter[billing_group_id],
    filter[emergency_address_id], filter[customer_reference]
    """

    billing_group_id: str
    """Identifies the billing group associated with the phone number."""

    connection_id: str
    """Identifies the connection associated with the phone number."""

    customer_reference: str
    """A customer reference string for customer look ups."""

    deletion_lock_enabled: bool
    """Indicates whether to enable or disable the deletion lock on each phone number.

    When enabled, this prevents the phone number from being deleted via the API or
    Telnyx portal.
    """

    external_pin: str
    """
    If someone attempts to port your phone number away from Telnyx and your phone
    number has an external PIN set, we will attempt to verify that you provided the
    correct external PIN to the winning carrier. Note that not all carriers
    cooperate with this security mechanism.
    """

    hd_voice_enabled: bool
    """Indicates whether to enable or disable HD Voice on each phone number.

    HD Voice is a paid feature and may not be available for all phone numbers, more
    details about it can be found in the Telnyx support documentation.
    """

    tags: SequenceNotStr[str]
    """A list of user-assigned tags to help organize phone numbers."""

    voice: UpdateVoiceSettingsParam


class FilterVoiceConnectionName(TypedDict, total=False):
    contains: str
    """Filter contains connection name. Requires at least three characters."""

    ends_with: str
    """Filter ends with connection name. Requires at least three characters."""

    eq: str
    """Filter by connection name."""

    starts_with: str
    """Filter starts with connection name. Requires at least three characters."""


class Filter(TypedDict, total=False):
    billing_group_id: str
    """Filter by the billing_group_id associated with phone numbers.

    To filter to only phone numbers that have no billing group associated them, set
    the value of this filter to the string 'null'.
    """

    connection_id: str
    """Filter by connection_id."""

    customer_reference: str
    """Filter numbers via the customer_reference set."""

    emergency_address_id: str
    """Filter by the emergency_address_id associated with phone numbers.

    To filter only phone numbers that have no emergency address associated with
    them, set the value of this filter to the string 'null'.
    """

    has_bundle: str
    """Filter by phone number that have bundles."""

    phone_number: str
    """Filter by phone number.

    Requires at least three digits. Non-numerical characters will result in no
    values being returned.
    """

    status: Literal[
        "purchase-pending",
        "purchase-failed",
        "port-pending",
        "active",
        "deleted",
        "port-failed",
        "emergency-only",
        "ported-out",
        "port-out-pending",
    ]
    """Filter by phone number status."""

    tag: str
    """Filter by phone number tags."""

    voice_connection_name: Annotated[FilterVoiceConnectionName, PropertyInfo(alias="voice.connection_name")]
    """Filter by voice connection name pattern matching."""

    voice_usage_payment_method: Annotated[
        Literal["pay-per-minute", "channel"], PropertyInfo(alias="voice.usage_payment_method")
    ]
    """Filter by usage_payment_method."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .porting_order_type import PortingOrderType

__all__ = ["PortingOrderMisc"]


class PortingOrderMisc(BaseModel):
    new_billing_phone_number: Optional[str] = None
    """New billing phone number for the remaining numbers.

    Used in case the current billing phone number is being ported to Telnyx. This
    will be set on your account with your current service provider and should be one
    of the numbers remaining on that account.
    """

    remaining_numbers_action: Optional[Literal["keep", "disconnect"]] = None
    """
    Remaining numbers can be either kept with their current service provider or
    disconnected. 'new_billing_telephone_number' is required when
    'remaining_numbers_action' is 'keep'.
    """

    type: Optional[PortingOrderType] = None
    """A port can be either 'full' or 'partial'.

    When type is 'full' the other attributes should be omitted.
    """
